#define _XOPEN_SOURCE 700
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>

#include <pwd.h>
#include <grp.h>

#include <stdint.h>
#include <time.h>

#define checkError(expr,userMsg) \
    do {\
        if (!(expr)) {\
            perror(userMsg);\
            exit(EXIT_FAILURE);\
        }\
    }  while (0)

#define MAX_SIZE (1024)

void myStat(char* path, char** pString) {

    struct stat fInfo;
    checkError(stat(path, &fInfo) != -1, "stat");

    *pString = malloc(sizeof(char)*MAX_SIZE);
    char* s = *pString;
    checkError(s != NULL, "malloc");

    char pravaPristupa[11] = "-rwxrwxrwx";
    // char* pravaPristupa = "-rwxrwxrwx";

    switch (fInfo.st_mode & S_IFMT) {
        case S_IFREG:
            pravaPristupa[0] = '-';
            break;
        case S_IFDIR:
            pravaPristupa[0] = 'd';
            break;
        case S_IFSOCK:
            pravaPristupa[0] = 's';
            break;
        case S_IFBLK:
            pravaPristupa[0] = 'b';
            break;
        case S_IFCHR:
            pravaPristupa[0] = 'c';
            break;
        case S_IFIFO:
            pravaPristupa[0] = 'p';
            break;
        case S_IFLNK:
            pravaPristupa[0] = 'l';
            break;
    }    

    if ((fInfo.st_mode & S_IRUSR) != S_IRUSR)
        pravaPristupa[1] = '-';
    if ((fInfo.st_mode & S_IWUSR) != S_IWUSR)
        pravaPristupa[2] = '-';
    if ((fInfo.st_mode & S_IXUSR) != S_IXUSR)
        pravaPristupa[3] = '-';

    if ((fInfo.st_mode & S_IRGRP) != S_IRGRP)
        pravaPristupa[4] = '-';
    if ((fInfo.st_mode & S_IWGRP) != S_IWGRP)
        pravaPristupa[5] = '-';
    if ((fInfo.st_mode & S_IXGRP) != S_IXGRP)
        pravaPristupa[6] = '-';

    if ((fInfo.st_mode & S_IROTH) != S_IROTH)
        pravaPristupa[7] = '-';
    if ((fInfo.st_mode & S_IWOTH) != S_IWOTH)
        pravaPristupa[8] = '-';
    if ((fInfo.st_mode & S_IXOTH) != S_IXOTH)
        pravaPristupa[9] = '-';

    int bytes = sprintf(s, "%s", pravaPristupa);

    struct passwd* pwd = getpwuid(fInfo.st_uid);
    checkError(pwd != NULL, "pwd");

    bytes += sprintf(s + bytes, " %s", pwd->pw_name);

    struct group* grp = getgrgid(fInfo.st_gid);
    checkError(grp != NULL, "grp");

    bytes += sprintf(s + bytes, " %s", grp->gr_name);

    bytes += sprintf(s + bytes, " %jd", (intmax_t)(fInfo.st_size));

    //char* s = ctime(fInfo.st_atim.tv_sec);
    char* vreme = ctime(&(fInfo.st_atime));
    bytes += sprintf(s + bytes, " %s", vreme);

    s[bytes-1] = 0;
    bytes--;

    bytes += sprintf(s + bytes, " %s", path);
}

int main(int argc, char** argv) {

    checkError(argc == 2, "args");

    char* s = NULL;
    myStat(argv[1], &s);

    printf("%s\n", s);
    free(s);

    exit(EXIT_SUCCESS);
}